;nyquist plug-in
;version 3
;type generate
;name "Append Import..."
;action "Importing, please wait..."
;info "by Steve Daulton (www.easyspacepro.com). GPL v2.\n* A stereo track must be selected for stereo output.\n* Supported file types: .WAV .AIFF .AU.\n"

;; AppendImport.ny by Steve Daulton. Aug 2011.
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

;control viewhelp "View Help" choice "No,Yes" 0
;control ImportMode "Import Type" choice "Sequential,Alternate Channels" 0  
;control channels "Output" choice "Mono,Stereo" 0
;control *default-sf-dir* "Import Folder" string "" "" 

(setq help  
"Append Import will look in the selected Import
Folder for a text file called 'filelist.txt'.
If found, it will attempt to read each audio file
that is listed in 'filelist.txt'.\n
filelist.txt must contain only a list of uncompressed
audio files of type .WAV, .AIFF or .AU. 
The file extension .AIF is also allowed.\n
Each file must be on a new line and must be either
1) A file in the 'Import Folder'
2) A fully qualified file name.\n
If 'filelist.txt' is not found, all supported audio
files in the Import Folder will be loaded in alpha-
numeric order.\n
Import Type:
'Sequential' will append the files in a single sequence.
'Alternate Channels' will assume that the first file
is intended for the left channel, the next file for
the right, and so on.\n
To see a list of imported file names, click the
Debug button.")

;ensure that path ends with separator
(setq *default-sf-dir* (strcat (string-right-trim " \\/" *default-sf-dir*)(string *file-separator*)))
(setq filelist (strcat *default-sf-dir* "filelist.txt"))

(defun convert (s-in)
	;get sample rate of s-in
	(setq s-in-srate 
		(if (arrayp s-in)
			(snd-srate (aref s-in 0))
			(snd-srate s-in)))
	(case channels
		;mono out
		(0 (if (arrayp s-in)
				(setq s-in (mult 0.5 (sim (aref s-in 0)(aref s-in 1))))))
		;stereo out
		(1 (if (not (arrayp s-in))
				(setq s-in (vector s-in s-in)))))
	;convert sample rate if necessary
	(if (/= s-in-srate *sound-srate*)
		(resample s-in *sound-srate*) s-in))

(defun test-ext (fnm); returns T for valid files
	(if (and (>= (length fnm) 5)
				(string-equal (subseq fnm (- (length fnm) 5)) ".aiff"))
		T
		(if (and (>= (length fnm) 4)
					(or (string-equal (subseq fnm (- (length fnm) 4)) ".wav")
							(string-equal (subseq fnm (- (length fnm) 4)) ".aif")))
			T
			(if (and (>= (length fnm) 3)	
						(string-equal (subseq fnm (- (length fnm) 3)) ".au"))
			T nil))))

;sort list
(defun sort (testl &aux newlist)
	(dotimes (j (length testl) newlist)	
		(let ((out	""))
			(dolist (i testl newlist)
				(setq out (if (string-greaterp out i) out i)))
			(setq newlist (cons out newlist))
			(setq testl (remove out testl)))))

;get all files in path in alphanumeric order
(defun get-all ()
	(let (outlist 
			(flst  (listdir *default-sf-dir*)))
		(dotimes (i (length flst))
			(if (test-ext (nth i flst))
				(setq outlist (cons (nth i flst) outlist))))
	 	(sort outlist)))

;print list of files
(defun pflist (fnamelist)
	(dolist (fname fnamelist)
		(format T "~a~%" fname)))

;split list alternate left/right
(defun split-alt (full-list)
	(let (Llist Rlist)
		(do ((i 0 (setq i (+ 2 i))))
				((= i (length full-list))(vector (reverse Llist) (reverse Rlist)))
			(setq Rlist (cons (nth i full-list)Rlist))
			(setq Llist (cons (nth (1+ i) full-list)Llist)))))

;If View Help
(if (> viewhelp 0)
	(format nil help)
	;ELSE check that input folder exists
	(if (listdir *default-sf-dir*)
		(progn
			;get file list
			(setq flist
				;if filelist.txt is found
				(if (setq fp (open filelist :direction :input))
					(do ((line (read-line fp)(setq line(read-line fp))) flist)
						 ((not line) (reverse flist))
						 (if (test-ext line)
						 	(setq flist (cons line flist))))
				;ELSE get all files in path
				(get-all)))

		;print file names to debug window
		(pflist flist)
		;get sounds from file list
		(setf slist (mapcar #'s-read flist))
		
		(if (= ImportMode 1); Alternate Channels
			(progn 
				;get left / right lists in an array
				(setq slist (split-alt slist))
				;set all imports to mono
				(setq channels 0)
				(setf Lslist (mapcar #'convert (aref slist 0)))
				(setf Rslist (mapcar #'convert (aref slist 1)))
				;create stereo sound
				(vector
					(seqrep (i (length (aref slist 0)))(cue (nth i (aref slist 0))))
					(seqrep (i (length (aref slist 1)))(cue (nth i (aref slist 1))))))
			
			(progn ; ELSE Sequential Append		
				;convert sounds if necessary
				(setf slist (mapcar #'convert slist))		
				;output sounds from list
				(seqrep (i (length slist))(cue (nth i slist))))))
		
	;OR output error
	(format nil "Error.~%Directory \"~a\" does not exist.~%" *default-sf-dir*)))
	

	





